"""
# Copyright (C) 2019 Zerodebug
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

2.3. Implement a set of parameters for controlling all tracks.
The example plug-in will consist of 10 mixer channels. Each channel will contain: volume, pan, sends, solo, mute, track name, record arm, automation state.

"""
import sys
if sys.version_info[0] >= 3:
    import struct
    import Live
    import time
    import _thread
    from threading import Timer
    from collections import defaultdict
    from .Logger import Logger
    from time import sleep
    from . import LiveUtils
else:
    import struct
    import Live
    import LiveUtils
    import time
    import unicodedata
    import thread
    from threading import Timer
    from collections import defaultdict
    from Logger import Logger
    from time import sleep


def repr_new(input_str):
    output_string = "unreadable_string"


    if input_str == None:
        return str(output_string)

    if sys.version_info[0] >= 3:


        try:
            output_string = input_str.encode('ascii', 'ignore')
            output_string1 = output_string.decode()
            return str(output_string1)
        except:
            pass


        try:
            output_string = input_str.encode("UTF-8")
            output_string1 = output_string.decode()
            return str(output_string1)
        except:
            pass

    else:

        try:
            output_string = input_str.encode('ascii', 'ignore')
            return str(output_string)
        except:
            pass

        try:
            output_string = input_str.encode("UTF-8")
            return str(output_string)
        except:
            pass

    return "unreadable_string"


def cut_string(input_str):
    info = (input_str[0:331] + '..') if len(input_str) > 333 else input_str
    return (info)



def sign_extend(bits, value):
    sign_bit = 1 << (bits - 1)
    return (value & (sign_bit - 1)) - (value & sign_bit)


def get_device_for_message(msg):

    type = msg[2]
    tid = msg[3]
    did = msg[4]
        
    number_of_steps = msg[5]
        
    if type == 0:
        track = LiveUtils.getSong().tracks[tid]
    elif type == 2:
        track = LiveUtils.getSong().return_tracks[tid]
    elif type == 1:
        track = LiveUtils.getSong().master_track

    device = track.devices[did]
        
    for i in range(number_of_steps):
        chain_id = msg[6+i*2]
        device_id = msg[7+i*2]
            
        track = device.chains[chain_id]
        device = track.devices[device_id]
        

    return device


def tuple_idx(tuple, obj):
    for i in range(0,len(tuple)):
        if (tuple[i] == obj):
            return i
    return -1
